class MissingParameter(Exception):
    """This error is raised when something goes wrong due to missing parameter"""

class MissingLabel(Exception):
    """This error is raised when something goes wrong due to missing labels"""

class MissingImport(Exception):
    """This error is raised when an import is missing but fail is not needed"""
